#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Doc here.
"""

import argparse
from collections import namedtuple
import logging
import platform
import subprocess

import nagiosplugin


__docformat__ = 'restructuredtext en'

_log = logging.getLogger('nagiosplugin')

fields = ('Neighbor', 'AS', 'MsgRcvd', 'MsgSent', 'OutQ', 'Up_Down',
          'State_PrfRcvd')

Session = namedtuple('Session', fields)


def _popen(cmd):  # pragma: no cover
    """Try catched subprocess.popen.

    raises explicit error
    """
    try:
        proc = subprocess.Popen(cmd,
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()
        return stdout, stderr

    except OSError as e:
        message = "%s" % e
        raise nagiosplugin.CheckError(message)


class CheckBgpCtl(nagiosplugin.Resource):
    """Check bgpctl sessions plugin."""

    hostname = platform.node()
    cmd = 'bgpctl show'

    def __init__(self, ignore_list):
        self.ignore_list = ignore_list

    def _get_sessions(self):
        """Runs 'bgpctl show'."""

        _log.debug("running '%s'", self.cmd)
        stdout, stderr = _popen(self.cmd.split())

        if not isinstance(stderr, str):  # pragma: no cover
            stderr = stderr.decode()
        if not isinstance(stdout, str):  # pragma: no cover
            stdout = stdout.decode()

        if stderr:
            message = "%s %s" % (self.hostname, stderr.splitlines()[-1])
            _log.info(message)
            raise nagiosplugin.CheckError(message)

        if stdout:
            output = stdout.splitlines()[1:]
            if output:
                return [Session(*line.rsplit(None, len(Session._fields) - 1))
                        for line in output]

    def check_session(self, session):
        """Check session is up, or not in ignore list."""
        result = -1
        state = session.State_PrfRcvd.split('/')[0]

        if state.isdigit():
            result = int(state)

        else:
            if self.ignore_list is not None:
                if session.Neighbor in self.ignore_list:
                    result = 0

        return result

    def probe(self):
        """."""
        self.sessions = self._get_sessions()
        if self.sessions:
            for session in self.sessions:
                yield nagiosplugin.Metric(session.Neighbor,
                                          self.check_session(session),
                                          min=0, context='bgpctl')


class AuditSummary(nagiosplugin.Summary):
    """Status line conveying informations."""

    def ok(self, results):
        """Summarize OK(s)."""
        return 'All bgp sessions in correct state'


def parse_args():  # pragma: no cover
    """Arguments parser."""
    argp = argparse.ArgumentParser(description=__doc__)
    argp.add_argument('-v', '--verbose', action='count', default=0,
                      help='increase output verbosity (use up to 3 times)')
    argp.add_argument('--ignore-list', nargs='*')

    return argp.parse_args()


@nagiosplugin.guarded
def main():  # pragma: no cover

    args = parse_args()
    check = nagiosplugin.Check(CheckBgpCtl(args.ignore_list),
                               nagiosplugin.ScalarContext('bgpctl', None,
                                                          '0:'),
                               AuditSummary())
    check.main(args.verbose)

# vim:set et sts=4 ts=4 tw=80:



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os
import sys


version = '0.10'

here = os.path.abspath(os.path.dirname(__file__))


def read_file(*pathes):
    path = os.path.join(here, *pathes)
    if os.path.isfile(path):
        with open(path, 'r') as desc_file:
            return desc_file.read()
    else:
        return ''


desc_files = (('README.rst',), ('docs', 'CHANGES.rst'),
              ('docs', 'CONTRIBUTORS.rst'))

long_description = '\n\n'.join([read_file(*pathes) for pathes in desc_files])

install_requires = ['nagiosplugin']

if sys.version_info < (2, 7):
    extras_require = {'test': ['setuptools', 'mock', 'unittest2', 'argparse']}
    install_requires.append('argparse')
else:
    extras_require = {'test': ['setuptools', 'mock']}

setup(name='checkopenbgpd',
      version=version,
      description="Check OpenBGPD sessions Nagios|Icinga|shinken|etc plugin",
      long_description=long_description,
      platforms=["any"],
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: POSIX :: BSD :: OpenBSD',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Monitoring',
        ],
      keywords="Nagios Icinga plugin check openbgpd openbsd monitoring",
      author="Jean-Philippe Camguilhem",
      author_email="pypi@camguilhem.net",
      url="https://github.com/jpcw/checkopenbgpd/",
      license="BSD",
      packages=find_packages("src"),
      package_dir={"": "src"},
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      test_suite='checkopenbgpd.tests',
      extras_require=extras_require,
      entry_points="""
      # -*- Entry points: -*-
      [console_scripts]
      check_openbgpd = checkopenbgpd:main
      """,
      )

# vim:set et sts=4 ts=4 tw=80:



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
checkopenbgpd
------------------------------
Check OpenBGPD sessions Nagios|Icinga|shinken|etc plugin

"""

__docformat__ = 'restructuredtext en'

from .checkopenbgpd import main  # NOQA

# vim:set et sts=4 ts=4 tw=80:




import mock

try:
    import unittest2 as unittest
except ImportError:  # pragma: no cover
    import unittest

import nagiosplugin
from nagiosplugin.metric import Metric
from nagiosplugin import CheckError

from checkopenbgpd import checkopenbgpd

no_sudoer = 'bgpctl: connect: /var/run/bgpd.sock: Permission denied'
no_bgpd = 'bgpctl: connect: /var/run/bgpd.sock: No such file or directory'


bgpctl_sh = 'Neighbor     AS MsgRcvd MsgSent OutQ Up/Down   State/PrfRcvd\n'\
            'FIRST     65001   75386       6     0 5d02h04m Idle\n'\
            'SECOND    65001 2832677  113152     0 5d01h58m 529001\n'\
            'THIRD     65002 3914143  103074     0 3d11h59m Idle\n'\
            'FOURTH    65222     281     278     0 02:16:21 3/20\n'\
            'FIFTH     65115       0       0     0 Never    Active\n'\
            'SIXTH     65115       0       0     0 Never    Active'


class Test_checkopenbgpd(unittest.TestCase):

    def test_bgpd_is_not_running(self):
        check = checkopenbgpd.CheckBgpCtl(None)
        err_message = no_bgpd

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = '', err_message
            with self.assertRaises(CheckError):
                check._get_sessions()  # NOQA

    def test_user_is_not_sudoer(self):
        check = checkopenbgpd.CheckBgpCtl(None)
        err_message = no_sudoer

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = '', err_message
            with self.assertRaises(CheckError):
                check._get_sessions()  # NOQA

    def test__get_sessions(self):
        check = checkopenbgpd.CheckBgpCtl(None)
        output = bgpctl_sh

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = output, ''
            sessions = check._get_sessions()
            self.assertEquals(len(sessions), 6)
            self.assertEquals(type(sessions[0]), checkopenbgpd.Session)

    def test_check_session_is_up(self):
        check = checkopenbgpd.CheckBgpCtl(None)
        output = bgpctl_sh

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = output, ''
            sessions = check._get_sessions()
            result = check.check_session(sessions[1])
            self.assertEquals(result, 529001)

    def test_check_ignore_session_in_ignore_list(self):
        check = checkopenbgpd.CheckBgpCtl(['THIRD', 'FIFTH'])
        output = bgpctl_sh

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = output, ''
            sessions = check._get_sessions()
            result = check.check_session(sessions[2])
            self.assertEquals(result, 0)

    def test_check_ignore_session_not_in_ignore_list(self):
        check = checkopenbgpd.CheckBgpCtl(['THIRD'])
        output = bgpctl_sh

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = output, ''
            sessions = check._get_sessions()
            result = check.check_session(sessions[0])
            self.assertEquals(result, -1)

    def test_check_probe_without_ignore_list(self):
        check = checkopenbgpd.CheckBgpCtl(None)
        output = bgpctl_sh

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = output, ''
            check._get_sessions()
            probe = check.probe()
            first = next(probe)
            self.assertEquals(type(first), Metric)
            self.assertEquals(first.value, -1)
            second = next(probe)
            self.assertEquals(second.value, 529001)
            third = next(probe)
            self.assertEquals(third.value, -1)
            fourth = next(probe)
            self.assertEquals(fourth.value, 3)
            fifth = next(probe)
            self.assertEquals(fifth.value, -1)
            sixth = next(probe)
            self.assertEquals(sixth.value, -1)

    def test_check_probe_with_ignore_list(self):
        check = checkopenbgpd.CheckBgpCtl(['THIRD', 'FIFTH'])
        output = bgpctl_sh

        with mock.patch("checkopenbgpd.checkopenbgpd._popen") as _popen:
            _popen.return_value = output, ''
            check._get_sessions()
            probe = check.probe()
            first = next(probe)
            self.assertEquals(type(first), Metric)
            self.assertEquals(first.value, -1)
            second = next(probe)
            self.assertEquals(second.value, 529001)
            third = next(probe)
            self.assertEquals(third.value, 0)
            fourth = next(probe)
            self.assertEquals(fourth.value, 3)
            fifth = next(probe)
            self.assertEquals(fifth.value, 0)
            sixth = next(probe)
            self.assertEquals(sixth.value, -1)


class Test_AuditSummary(unittest.TestCase):

    def test_ok(self):
        from nagiosplugin.result import Result, Results
        from nagiosplugin.state import Ok
        from checkopenbgpd.checkopenbgpd import AuditSummary
        results = Results()
        ok_r1 = Result(Ok, '', nagiosplugin.Metric('met1', 529001))
        ok_r2 = Result(Ok, '', nagiosplugin.Metric('met1', 0))
        ok_r3 = Result(Ok, '', nagiosplugin.Metric('met1', 3))
        results.add(ok_r1)
        results.add(ok_r2)
        results.add(ok_r3)
        summary = AuditSummary()
        sum_ok = summary.ok(results)
        self.assertEquals(sum_ok, 'All bgp sessions in correct state')
